#!/usr/bin/env python
"""analytical solution to NPBE for concentric spheres and dipole

   based on Rochia et al, JPC B, 105:6507, 2001

   Copyright 2005, Mark Engelhardt (marke@stanford.edu)
"""
from math import pi,exp,cos,sqrt

# CONSTANTS
e0 = 1.6022e-19 #electron charge, C
kb = 1.3806503e-23 #J K-1 boltzmann constant
Na = 6.022e23 # mol-1
R = 8.314 # J K-1 mol-1
E0 = 8.8542e-12 # C2 J-1 m-1
Dconv = 3.33564e-30 * 4.803 # C m
T = 298.

kbTe = kb*T/e0

def pot(r,phi,P,e1,e2,e3,a,b,kap):
  """calculates the potential at a point (kT/e)

  r - distance from the center (m)
  phi - angle to the dipole (radians)
  P - dipole (Debye)
  e1, e2, e3 - the dielectric in the 3 concentric spheres
  a, b - the radii of the inner 2 spheres
  kap - the screening constant
  """
  
  r /= 1.e10
  D = (b/a)**3.
  A = D*(2.*e2+e1)/(e1-e2)
  B = 1. + (A+2.)/(1.-A)
  C = e2*(kap*b+1.)*(2.+A)/(1.-A)-e3*(2.+2.*kap*b+(kap*b)**2)
  pot = 3.*P*e2/(e1-e2)*D*B/C*(kap*r+1)*exp(-kap*(r-b))*cos(phi)/r**2
  # note the factor of 4
  pot /= 4*pi
  return pot/kbTe

def kappa(conc,eps,temp=298.):
  """calculates the screening constant for the solution

  conc - monovalent ion concentration
  eps - dielectric constant (1 is vacuum)
  temp - the temperature
  """
  
  k = sqrt(1000.*2.*e0**2.*conc*Na**2./eps/temp/R)
  return k

if __name__ == "__main__":
  # define the 3 dielectric constants
  e1 = 4.0 * E0 # C2 J-1 m-1
  e2 = 20.0 * E0
  e3 = 80.0 * E0
  
  # define the radii of the inner 2 spheres
  a = 4.e-10 # m
  b = 6.e-10

  # define the concentration of monovalent ions
  conc = 0.0

  # define the angle to the dipole and the dipole
  ###  careful - the unit "Debye" has multiple definitions, and the authors
  ###  do not appear to use the most commonly accepted definition
  phi = pi/4. # radians
  dipole = 4.* Dconv # C m
  dipole = 4.*e0*1e-10 # C m - charge of +4 x e0 coulombs x 1 angstrom

  # calculate the potential at points along the line defined by the above
  for j in range(12,40):
    i = float(j)/2
    print i,
    print pot(i,phi,dipole,e1,e2,e3,a,b,kappa(conc,e3)),
    print pot(i,phi,dipole,1.*E0,e2,e3,a,b,kappa(conc,e3)),
    print pot(i,phi,dipole,e1,e2,e3,a,b,kappa(0.1,e3))
  ## repeat for 14.14, which is the point they use in their paper
  print 14.14,
  print pot(14.14,phi,dipole,e1,e2,e3,a,b,kappa(conc,e3)),
  print pot(14.14,phi,dipole,1.*E0,e2,e3,a,b,kappa(conc,e3)),
  print pot(14.14,phi,dipole,e1,e2,e3,a,b,kappa(0.1,e3))

