#!/usr/bin/env python
"""A class and functions to deal with dx grid files

   intended primarily for ISIM potential files

   Copyright 2005, Mark Engelhardt (marke@stanford.edu)
"""
from math import ceil, floor
import sys
from sys import stderr, stdout

def Distance(coords1, coords2):
  """calculates the distance between two points in 3d space
  """
  x1 = coords1[0]
  y1 = coords1[1]
  z1 = coords1[2]
  x2 = coords2[0]
  y2 = coords2[1]
  z2 = coords2[2]
  return ((x1-x2)**2+(y1-y2)**2+(z1-z2)**2)**0.5

def SortCoordList(p,r):
  """takes p and r, sets of coords, and returns them in x,y,z priority
  """
  for i in range(len(p)):
    if p[i] > r[i]:
      return 1
    elif r[i] > p[i]:
      return -1
  return 0

class pot_grid:
  """defines a grid of x,y,z,pot (a list) and associated functions
  """
  def __init__(self, filename):
    """reads in the grid values from a file

    self.grid is a list of points (also lists) x,y,z,potential
    the isim grid is NOT PRE-SORTED
    """
    self.x_vals = []
    self.y_vals = []
    self.z_vals = []
    
    self.grid = open(filename).readlines() # this will be a huge memory chunk
    for i in range(len(self.grid)):
      self.grid[i] = [float(item) for item in self.grid[i].split()]
    print 'done reading grid file'
 
  def PotLookup(self, coords, cutoff = 3):
    """returns the potential at x,y,z, from an average of the surrounding points

    slow - calculates the distance between the point at coords and every other
    point
    """
    total_weights = 0
    sum_inverses = 0
    i = 0
    for point in self.grid:
      i += 1
      if abs(coords[0]-point[0]) > cutoff: continue
      if abs(coords[1]-point[1]) > cutoff: continue
      if abs(coords[2]-point[2]) > cutoff: continue
      d = Distance(coords, point[:3])
      if d <= cutoff:
        w = 1/d**2
        total_weights += w
        sum_inverses += w * point[3]
    print "summing over %d points" % i
    if total_weights == 0:
      stderr.write("returning 0 from pot lookup - no valid points\n")
      return 0.0
    return sum_inverses / total_weights
 
  def GetLoHi(self, list, val, hi_index, lo_index=0):
    """returns a lists that bracket the value given for the chosen dimension

    a recursive function
    returns 2 lists, one values below the value, and one for the step above
    the coordinates in the returned lists are shifted
    returns two empty arrays if it runs off the end of the list
    list - a list of coordinate, ... information
    val - the value you wish to bracket
    pos - the index of the array
    """
    if not list:
      print 'GOT EMPTY LIST'
      return ([],[])
    pos = (lo_index + hi_index) / 2
    if pos < 0 or pos >= len(list):
      raise "pos is out of range"
    
    # running off the list: return the whole list?
    if list[pos][0] < val and pos == len(list)-1:
      print "ran off the top of the list, val (%8.3f) \
          list[0][0] (%8.3f) list[pos][0] (%8.3f)"\
          % (val, list[0][0], list[pos][0])
      print "list length is: (length %d)" % len(list)
#      print list
      raise
      return (list,list)

    # found the exact value requested, returns identical lists
    elif list[pos][0] == val and (pos == 0 or list[pos-1][0] < val):
      # special case, return the same array for hi and low
      arr = []
      for i in range(pos, len(list)): # check this - including the last el?
        if list[i][0] > val:
          break
        arr.append(list[i][1:]+list[i][0:1]) # shuffle front coord to back
      return (arr, arr)

    # found a boundary
    elif list[pos][0] > val and (pos == 0 or list[pos-1][0] < val):
      hi = list[pos][0]
      if pos == 0:
        lo = 'undefined'
      else:
        lo = list[pos-1][0]
      hi_arr = []
      lo_arr = []
      for i in range(pos,len(list)): # check this - including the last el?
        if list[i][0] > hi:
          break
        hi_arr.append(list[i][1:]+list[i][0:1]) # shuffle front coord to back
      for i in range(pos-1,-1,-1): # goes down to position zero in the list
        if list[i][0] < lo:
          break
        lo_arr.append(list[i][1:]+list[i][0:1])
      lo_arr.reverse() # because a -1 increment was used to generate the list
      print 'returning 2 arrays from a boundary'
      print lo_arr
      print '-\n', hi_arr
      return (lo_arr, hi_arr)

    # too high, go higher
    elif list[pos][0] < val:
      return self.GetLoHi(list, val, hi_index, pos + 1)

    # too low, go lower
    else:
      return self.GetLoHi(list, val, pos-1, lo_index)

if __name__=="__main__":
  infile = 'GRID_POT_TEST'
  coords = [-.33322, -.00873, -46.65]
  g = pot_grid(infile)
  pot = g.PotLookup(coords)
  print pot

